# Generated by Django 3.2.16 on 2022-10-17 09:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_cloudspotlicense', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='cloudspotcompany',
            name='users',
            field=models.ManyToManyField(to='django_cloudspotlicense.CloudspotUser'),
        ),
    ]
